using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace LightTypes
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        KeyboardState lastKbState;

        Model walls;
        Model columns;

        Effect[] lightEffect = new Effect[1];
        int currentEffect = 0;

        Vector3 cameraPos = new Vector3(0, 2, 4.8f);
        Vector3 lightPosition;

        Matrix view, projection;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            lightEffect[0] = Content.Load<Effect>("PointLight");

            walls = Content.Load<Model>("BathHouse-Walls");
            columns = Content.Load<Model>("BathHouse-Cols");

            view = Matrix.CreateLookAt(cameraPos, cameraPos - Vector3.UnitZ, Vector3.Up);
            float aspectRatio = (float)graphics.GraphicsDevice.Viewport.Width / (float)graphics.GraphicsDevice.Viewport.Height;
            projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(75), aspectRatio, 0.1f, 100);

            lightEffect[0].Parameters["View"].SetValue(view);
            lightEffect[0].Parameters["Projection"].SetValue(projection);

            lightEffect[0].Parameters["cameraPosition"].SetValue(cameraPos);
            lightEffect[0].Parameters["specularPower"].SetValue(50);
            lightEffect[0].Parameters["specularIntensity"].SetValue(0);

            lightPosition = new Vector3(0, 2.5f, 0);
            //---
            lightEffect[0].Parameters["lightPosition"].SetValue(lightPosition);
            SetLightColors(lightEffect[0]);

            //--
            ChangeEffect(walls, lightEffect[currentEffect]);
            ChangeEffect(columns, lightEffect[currentEffect]);
        }

        private void SetLightColors(Effect effect)
        {
            effect.Parameters["ambientLightColor"].SetValue(new Vector4(0.1f, 0.1f, 0.1f, 1));
            effect.Parameters["diffuseLightColor"].SetValue(new Vector4(0.5f, 0.5f, 0.5f, 1));
            effect.Parameters["specularLightColor"].SetValue(new Vector4(1.0f, 1.0f, 1.0f, 1));
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            KeyboardState kbState = Keyboard.GetState();

            // Allows the game to exit
            if (kbState.IsKeyDown(Keys.Escape))
                this.Exit();

            if (kbState.IsKeyDown(Keys.Space) && !lastKbState.IsKeyDown(Keys.Space))
            {
                currentEffect = (currentEffect + 1) % lightEffect.Length;
                ChangeEffect(walls, lightEffect[currentEffect]);
                ChangeEffect(columns, lightEffect[currentEffect]);
            }

            lastKbState = kbState;

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            DrawModel(walls, Matrix.Identity);
            DrawModel(columns, Matrix.Identity);

            base.Draw(gameTime);
        }

        private void DrawModel(Model model, Matrix world)
        {
            Matrix[] transforms = new Matrix[model.Bones.Count];
            model.CopyAbsoluteBoneTransformsTo(transforms);

            foreach (ModelMesh mesh in model.Meshes)
            {
                foreach (Effect effect in mesh.Effects)
                {
                    effect.Parameters["World"].SetValue(transforms[mesh.ParentBone.Index] * world);
                }
                mesh.Draw();
            }
        }

        public void ChangeEffect(Model model, Effect effect)
        {
            foreach (ModelMesh mesh in model.Meshes)
            {
                foreach (ModelMeshPart part in mesh.MeshParts)
                {
                    part.Effect = effect;
                }
            }
        }
    }
}
